@@ -13,7 +13,7 @@
 
 #include <QtXml/QDomDocument>
 
-namespace CalamaresUtils
+namespace Calamares
 {
 namespace GeoIP
 {
@@ -59,7 +59,6 @@ getElementTexts( const QByteArray& data, const QString& tag )
     return elements;
 }
 
-
 QString
 GeoIPXML::rawReply( const QByteArray& data )
 {
@@ -90,4 +89,4 @@ GeoIPXML::processReply( const QByteArray& data )
 }
 
 }  // namespace GeoIP
-}  // namespace CalamaresUtils
+}  // namespace Calamares
