@@ -42,14 +42,16 @@ enum class Interface
     QtPlugin,  // Jobs or Views
     Python,  // Jobs only
     Process,  // Deprecated interface
-    PythonQt  // Views only, available as enum even if PythonQt isn't used
 };
 const NamedEnumTable< Interface >& interfaceNames();
 
 
-/* While this isn't a useful *using* right now, the intention is
- * to create a more strongly-typed Module Descriptor that carries
- * only the necessary information and no variants.
+/**
+ * @brief Description of a module (obtained from module.desc)
+ *
+ * Provides access to the fields of a descriptor, use type() to
+ * determine which specialized fields make sense for a given
+ * descriptor (e.g. a Python module has no shared-library path).
  */
 class Descriptor
 {
@@ -112,10 +114,7 @@ public:
      *
      * Python job modules have one specific script to load and run.
      */
-    QString script() const
-    {
-        return ( m_interface == Interface::Python || m_interface == Interface::PythonQt ) ? m_script : QString();
-    }
+    QString script() const { return m_interface == Interface::Python ? m_script : QString(); }
 
 private:
     QString m_name;
