@@ -16,6 +16,7 @@
 #include <QDebug>
 #include <QObject>
 #include <QUrl>
+#include <QVector>
 
 #include <chrono>
 #include <memory>
@@ -23,7 +24,7 @@
 class QNetworkReply;
 class QNetworkRequest;
 
-namespace CalamaresUtils
+namespace Calamares
 {
 namespace Network
 {
@@ -89,16 +90,10 @@ class DLLEXPORT Manager : public QObject
 {
     Q_OBJECT
     Q_PROPERTY( bool hasInternet READ hasInternet NOTIFY hasInternetChanged FINAL )
-
-    Manager();
+    Q_PROPERTY( QVector< QUrl > checkInternetUrls READ getCheckInternetUrls WRITE setCheckHasInternetUrl )
 
 public:
-    /** @brief Gets the single Manager instance.
-     *
-     * Typical code will use `auto& nam = Manager::instance();`
-     * to keep the reference.
-     */
-    static Manager& instance();
+    Manager();
     ~Manager() override;
 
     /** @brief Checks if the given @p url returns data.
@@ -119,9 +114,6 @@ public:
      */
     QByteArray synchronousGet( const QUrl& url, const RequestOptions& options = RequestOptions() );
 
-    /// @brief Set the URL which is used for the general "is there internet" check.
-    void setCheckHasInternetUrl( const QUrl& url );
-
     /** @brief Do a network request asynchronously.
      *
      * Returns a pointer to the reply-from-the-request.
@@ -130,6 +122,18 @@ public:
      */
     QNetworkReply* asynchronousGet( const QUrl& url, const RequestOptions& options = RequestOptions() );
 
+    /// @brief Set the URL which is used for the general "is there internet" check.
+    static void setCheckHasInternetUrl( const QUrl& url );
+
+    /// @brief Adds an (extra) URL to check
+    static void addCheckHasInternetUrl( const QUrl& url );
+
+    /// @brief Set a collection of URLs used for the general "is there internet" check.
+    static void setCheckHasInternetUrl( const QVector< QUrl >& urls );
+
+    /// @brief What URLs are used to check for internet connectivity?
+    static QVector< QUrl > getCheckInternetUrls();
+
 public Q_SLOTS:
     /** @brief Do an explicit check for internet connectivity.
      *
@@ -156,8 +160,7 @@ signals:
 
 private:
     class Private;
-    std::unique_ptr< Private > d;
 };
 }  // namespace Network
-}  // namespace CalamaresUtils
+}  // namespace Calamares
 #endif  // LIBCALAMARES_NETWORK_MANAGER_H
