@@ -24,14 +24,19 @@
 
 class Device;
 class Partition;
+class PartitionTable;
 
-namespace CalamaresUtils
+namespace Calamares
 {
 namespace Partition
 {
 
 using ::Device;
 using ::Partition;
+using ::PartitionTable;
+
+/** @brief Get partition table */
+const PartitionTable* getPartitionTable( const Partition* partition );
 
 /** @brief Is this a free-space area? */
 bool isPartitionFreeSpace( const Partition* );
@@ -65,6 +70,6 @@ Partition* findPartitionByPath( const QList< Device* >& devices, const QString&
 QList< Partition* > findPartitions( const QList< Device* >& devices,
                                     std::function< bool( Partition* ) > criterionFunction );
 }  // namespace Partition
-}  // namespace CalamaresUtils
+}  // namespace Calamares
 
 #endif  // PARTITION_PARTITIONQUERY_H
