@@ -1,22 +1,10 @@
-/* === This file is part of Calamares - <https://github.com/calamares> ===
- * 
+/* === This file is part of Calamares - <https://calamares.io> ===
+ *
  *   SPDX-FileCopyrightText: 2018 Adriaan de Groot <groot@kde.org>
- *
- *   Calamares is free software: you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation, either version 3 of the License, or
- *   (at your option) any later version.
- *
- *   Calamares is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with Calamares. If not, see <http://www.gnu.org/licenses/>.
- *
  *   SPDX-License-Identifier: GPL-3.0-or-later
- *   License-Filename: LICENSE
+ *
+ *   Calamares is Free Software: see the License-Identifier above.
+ *
  *
  */
 
@@ -29,31 +17,35 @@
 #include <QVariant>
 
 #include <chrono>
+#include <utility>
 
-namespace CalamaresUtils
+class KMacroExpanderBase;
+
+namespace Calamares
 {
+using CommandLineBase = std::pair< QString, std::chrono::seconds >;
 
 /**
  * Each command can have an associated timeout in seconds. The timeout
  * defaults to 10 seconds. Provide some convenience naming and construction.
  */
-struct CommandLine : public QPair< QString, std::chrono::seconds >
+struct CommandLine : public CommandLineBase
 {
     static inline constexpr std::chrono::seconds TimeoutNotSet() { return std::chrono::seconds( -1 ); }
 
     /// An invalid command line
     CommandLine()
-        : QPair( QString(), TimeoutNotSet() )
+        : CommandLineBase( QString(), TimeoutNotSet() )
     {
     }
 
     CommandLine( const QString& s )
-        : QPair( s, TimeoutNotSet() )
+        : CommandLineBase( s, TimeoutNotSet() )
     {
     }
 
     CommandLine( const QString& s, std::chrono::seconds t )
-        : QPair( s, t )
+        : CommandLineBase( s, t )
     {
     }
 
@@ -62,6 +54,20 @@ struct CommandLine : public QPair< QString, std::chrono::seconds >
     std::chrono::seconds timeout() const { return second; }
 
     bool isValid() const { return !first.isEmpty(); }
+
+    /** @brief Returns a copy of this one command, with variables expanded
+     *
+     * The given macro-expander is used to expand the command-line.
+     * This will normally be a Calamares::String::DictionaryExpander
+     * instance, which handles the ROOT and USER variables.
+     */
+    CommandLine expand( KMacroExpanderBase& expander ) const;
+    /** @brief As above, with a default macro-expander.
+     *
+     * The default macro-expander assumes RunInHost (e.g. ROOT will
+     * expand to the RootMountPoint set in Global Storage).
+     */
+    CommandLine expand() const;
 };
 
 /** @brief Abbreviation, used internally. */
@@ -81,7 +87,6 @@ public:
     /** @brief empty command-list with timeout to apply to entries. */
     CommandList( bool doChroot = true, std::chrono::seconds timeout = std::chrono::seconds( 10 ) );
     CommandList( const QVariant& v, bool doChroot = true, std::chrono::seconds timeout = std::chrono::seconds( 10 ) );
-    ~CommandList();
 
     bool doChroot() const { return m_doChroot; }
 
@@ -93,15 +98,26 @@ public:
     using CommandList_t::const_iterator;
     using CommandList_t::count;
     using CommandList_t::isEmpty;
+    using CommandList_t::push_back;
+    using CommandList_t::value_type;
 
-protected:
-    using CommandList_t::append;
-    void append( const QString& );
+    /** @brief Return a copy of this command-list, with variables expanded
+     *
+     * Each command-line in the list is expanded with the given @p expander.
+     * @see CommandLine::expand() for details.
+     */
+    CommandList expand( KMacroExpanderBase& expander ) const;
+    /** @brief As above, with a default macro-expander.
+     *
+     * Each command-line in the list is expanded with that default macro-expander.
+     * @see CommandLine::expand() for details.
+     */
+    CommandList expand() const;
 
 private:
     bool m_doChroot;
     std::chrono::seconds m_timeout;
 };
 
-}  // namespace CalamaresUtils
+}  // namespace Calamares
 #endif
