@@ -12,8 +12,8 @@
 #define UTILS_RETRANSLATOR_H
 
 #include "DllMacro.h"
+#include "locale/Translation.h"
 
-#include <QList>
 #include <QObject>
 #include <QString>
 
@@ -23,14 +23,18 @@ class QEvent;
 class QLocale;
 class QTranslator;
 
-namespace CalamaresUtils
+namespace Calamares
 {
-/**
- * @brief installTranslator changes the application language.
- * @param locale the new locale.
+/** @brief changes the application language.
+ * @param locale the new locale (names as defined by Calamares).
  * @param brandingTranslationsPrefix the branding path prefix, from Calamares::Branding.
  */
-DLLEXPORT void installTranslator( const QLocale& locale, const QString& brandingTranslationsPrefix );
+DLLEXPORT void installTranslator( const Calamares::Locale::Translation::Id& locale,
+                                  const QString& brandingTranslationsPrefix );
+
+/** @brief Initializes the translations with the current system settings
+ */
+DLLEXPORT void installTranslator();
 
 /** @brief The name of the (locale of the) most recently installed translator
  *
@@ -38,7 +42,7 @@ DLLEXPORT void installTranslator( const QLocale& locale, const QString& branding
  * QLocale passed in, because Calamares will munge some names and
  * may remap translations.
  */
-DLLEXPORT QString translatorLocaleName();
+DLLEXPORT Calamares::Locale::Translation::Id translatorLocaleName();
 
 /** @brief Loads <prefix><locale> translations into the given @p translator
  *
@@ -53,7 +57,8 @@ DLLEXPORT QString translatorLocaleName();
  *
  * @returns @c true on success
  */
-DLLEXPORT bool loadTranslator( const QLocale& locale, const QString& prefix, QTranslator* translator );
+DLLEXPORT bool
+loadTranslator( const Calamares::Locale::Translation::Id& locale, const QString& prefix, QTranslator* translator );
 
 /** @brief Set @p allow to true to load translations from current dir.
  *
@@ -64,7 +69,6 @@ DLLEXPORT bool loadTranslator( const QLocale& locale, const QString& prefix, QTr
  */
 DLLEXPORT void setAllowLocalTranslation( bool allow );
 
-
 /** @brief Handles change-of-language events
  *
  * There is one single Retranslator object. Use `instance()` to get it.
@@ -85,7 +89,7 @@ public:
     static Retranslator* instance();
 
     /// @brief Helper function for attaching lambdas
-    static void attach( QObject* o, std::function< void( void ) > f);
+    static void attach( QObject* o, std::function< void( void ) > f );
 
 signals:
     void languageChanged();
@@ -97,8 +101,7 @@ private:
     explicit Retranslator( QObject* parent );
 };
 
-
-}  // namespace CalamaresUtils
+}  // namespace Calamares
 
 /** @brief Call code for this object when language changes
  *
@@ -111,7 +114,7 @@ private:
  *       immediately after setting up the connection. This allows
  *       setup and translation code to be mixed together.
  */
-#define CALAMARES_RETRANSLATE( body ) CalamaresUtils::Retranslator::attach( this, [=] { body } )
+#define CALAMARES_RETRANSLATE( body ) Calamares::Retranslator::attach( this, [ = ] { body } )
 /** @brief Call code for the given object (widget) when language changes
  *
  * This is identical to CALAMARES_RETRANSLATE, except the @p body is called
@@ -121,7 +124,7 @@ private:
  *       immediately after setting up the connection. This allows
  *       setup and translation code to be mixed together.
  */
-#define CALAMARES_RETRANSLATE_FOR( object, body ) CalamaresUtils::Retranslator::attach( object, [=] { body } )
+#define CALAMARES_RETRANSLATE_FOR( object, body ) Calamares::Retranslator::attach( object, [ = ] { body } )
 /** @brief Call a slot in this object when language changes
  *
  * Given a slot (in method-function-pointer notation), call that slot when the
@@ -135,8 +138,8 @@ private:
 #define CALAMARES_RETRANSLATE_SLOT( slotfunc ) \
     do \
     { \
-        connect( CalamaresUtils::Retranslator::instance(), &CalamaresUtils::Retranslator::languageChanged, this, slotfunc ); \
-        (this->*slotfunc)(); \
+        connect( Calamares::Retranslator::instance(), &Calamares::Retranslator::languageChanged, this, slotfunc ); \
+        ( this->*slotfunc )(); \
     } while ( false )
 
 #endif
