@@ -9,12 +9,14 @@
  *
  */
 
-#include "CalamaresUtilsSystem.h"
+#include "CommandList.h"
 #include "Entropy.h"
 #include "Logger.h"
 #include "RAII.h"
 #include "Runner.h"
 #include "String.h"
+#include "StringExpander.h"
+#include "System.h"
 #include "Traits.h"
 #include "UMask.h"
 #include "Variant.h"
@@ -22,6 +24,7 @@
 
 #include "GlobalStorage.h"
 #include "JobQueue.h"
+#include "compat/Variant.h"
 
 #include <QTemporaryFile>
 
@@ -45,7 +48,10 @@ private Q_SLOTS:
     void testLoadSaveYaml();  // Just settings.conf
     void testLoadSaveYamlExtended();  // Do a find() in the src dir
 
+    /** @section Test running commands and command-expansion. */
     void testCommands();
+    void testCommandExpansion_data();
+    void testCommandExpansion();  // See also shellprocess tests
 
     /** @section Test that all the UMask objects work correctly. */
     void testUmask();
@@ -75,6 +81,10 @@ private Q_SLOTS:
     void testStringRemoveTrailing_data();
     void testStringRemoveTrailing();
 
+    /** @section Test String expansion. */
+    void testStringMacroExpander_data();
+    void testStringMacroExpander();  // The KF5::CoreAddons bits
+
     /** @section Test Runner directory-manipulation. */
     void testRunnerDirs();
     void testCalculateWorkingDirectory();
@@ -94,6 +104,16 @@ LibCalamaresTests::~LibCalamaresTests() {}
 void
 LibCalamaresTests::initTestCase()
 {
+    Calamares::GlobalStorage* gs
+        = Calamares::JobQueue::instance() ? Calamares::JobQueue::instance()->globalStorage() : nullptr;
+
+    if ( !gs )
+    {
+        cDebug() << "Creating new JobQueue";
+        (void)new Calamares::JobQueue();
+        gs = Calamares::JobQueue::instance() ? Calamares::JobQueue::instance()->globalStorage() : nullptr;
+    }
+    QVERIFY( gs );
 }
 
 void
@@ -130,9 +150,9 @@ LibCalamaresTests::testLoadSaveYaml()
     cDebug() << QDir().absolutePath() << f.fileName() << f.exists();
     QVERIFY( f.exists() );
 
-    auto map = CalamaresUtils::loadYaml( f.fileName() );
+    auto map = Calamares::YAML::load( f.fileName() );
     QVERIFY( map.contains( "sequence" ) );
-    QCOMPARE( map[ "sequence" ].type(), QVariant::List );
+    QCOMPARE( Calamares::typeOf( map[ "sequence" ] ), Calamares::ListVariantType );
 
     // The source-repo example `settings.conf` has a show and an exec phase
     auto sequence = map[ "sequence" ].toList();
@@ -140,14 +160,14 @@ LibCalamaresTests::testLoadSaveYaml()
     for ( const auto& v : sequence )
     {
         cDebug() << Logger::SubEntry << v;
-        QCOMPARE( v.type(), QVariant::Map );
+        QCOMPARE( Calamares::typeOf( v ), Calamares::MapVariantType );
         QVERIFY( v.toMap().contains( "show" ) || v.toMap().contains( "exec" ) );
     }
 
-    CalamaresUtils::saveYaml( "out.yaml", map );
+    Calamares::YAML::save( "out.yaml", map );
 
-    auto other_map = CalamaresUtils::loadYaml( "out.yaml" );
-    CalamaresUtils::saveYaml( "out2.yaml", other_map );
+    auto other_map = Calamares::YAML::load( "out.yaml" );
+    Calamares::YAML::save( "out2.yaml", other_map );
     QCOMPARE( map, other_map );
 
     QFile::remove( "out.yaml" );
@@ -163,7 +183,9 @@ findConf( const QDir& d )
         QString path = d.absolutePath();
         path.append( d.separator() );
         for ( const auto& confname : d.entryList( { "*.conf" } ) )
+        {
             mine.append( path + confname );
+        }
         for ( const auto& subdirname : d.entryList( QDir::AllDirs | QDir::NoDotAndDotDot ) )
         {
             QDir subdir( d );
@@ -202,7 +224,6 @@ LibCalamaresTests::recursiveCompareMap( const QVariantMap& a, const QVariantMap&
     }
 }
 
-
 void
 LibCalamaresTests::testLoadSaveYamlExtended()
 {
@@ -212,10 +233,10 @@ LibCalamaresTests::testLoadSaveYamlExtended()
     {
         loaded_ok = true;
         cDebug() << "Testing" << confname;
-        auto map = CalamaresUtils::loadYaml( confname, &loaded_ok );
+        auto map = Calamares::YAML::load( confname, &loaded_ok );
         QVERIFY( loaded_ok );
-        QVERIFY( CalamaresUtils::saveYaml( "out.yaml", map ) );
-        auto othermap = CalamaresUtils::loadYaml( "out.yaml", &loaded_ok );
+        QVERIFY( Calamares::YAML::save( "out.yaml", map ) );
+        auto othermap = Calamares::YAML::load( "out.yaml", &loaded_ok );
         QVERIFY( loaded_ok );
         QCOMPARE( map.keys(), othermap.keys() );
         recursiveCompareMap( map, othermap, 0 );
@@ -227,7 +248,7 @@ LibCalamaresTests::testLoadSaveYamlExtended()
 void
 LibCalamaresTests::testCommands()
 {
-    using CalamaresUtils::System;
+    using Calamares::System;
     auto r = System::runCommand( System::RunLocation::RunInHost, { "/bin/ls", "/tmp" } );
 
     QVERIFY( r.getExitCode() == 0 );
@@ -251,6 +272,34 @@ LibCalamaresTests::testCommands()
     QVERIFY( r.getOutput().contains( tfn.fileName() ) );
 }
 
+void
+LibCalamaresTests::testCommandExpansion_data()
+{
+    QTest::addColumn< QString >( "command" );
+    QTest::addColumn< QString >( "expected" );
+
+    QTest::newRow( "empty" ) << QString() << QString();
+    QTest::newRow( "ls   " ) << QStringLiteral( "ls" ) << QStringLiteral( "ls" );
+    QTest::newRow( "user " ) << QStringLiteral( "chmod $USER" ) << QStringLiteral( "chmod alice" );
+}
+
+void
+LibCalamaresTests::testCommandExpansion()
+{
+    Calamares::GlobalStorage* gs
+        = Calamares::JobQueue::instance() ? Calamares::JobQueue::instance()->globalStorage() : nullptr;
+    QVERIFY( gs );
+    gs->insert( QStringLiteral( "username" ), QStringLiteral( "alice" ) );
+
+    QFETCH( QString, command );
+    QFETCH( QString, expected );
+    Calamares::CommandLine c( command, std::chrono::seconds( 0 ) );
+    Calamares::CommandLine e = c.expand();
+
+    QCOMPARE( c.command(), command );
+    QCOMPARE( e.command(), expected );
+}
+
 void
 LibCalamaresTests::testUmask()
 {
@@ -261,13 +310,13 @@ LibCalamaresTests::testUmask()
 
     // m gets the previous value of the mask (depends on the environment the
     // test is run in, might be 002, might be 077), ..
-    mode_t m = CalamaresUtils::setUMask( 022 );
-    QCOMPARE( CalamaresUtils::setUMask( m ), mode_t( 022 ) );  // But now most recently set was 022
+    mode_t m = Calamares::setUMask( 022 );
+    QCOMPARE( Calamares::setUMask( m ), mode_t( 022 ) );  // But now most recently set was 022
 
     for ( mode_t i = 0; i <= 0777 /* octal! */; ++i )
     {
         QByteArray name = ( ft.fileName() + QChar( '.' ) + QString::number( i, 8 ) ).toLatin1();
-        CalamaresUtils::UMask um( i );
+        Calamares::UMask um( i );
         int fd = creat( name, 0777 );
         QVERIFY( fd >= 0 );
         close( fd );
@@ -277,8 +326,8 @@ LibCalamaresTests::testUmask()
         QCOMPARE( mystat.st_mode & 0777, 0777 & ~i );
         QCOMPARE( unlink( name ), 0 );
     }
-    QCOMPARE( CalamaresUtils::setUMask( 022 ), m );
-    QCOMPARE( CalamaresUtils::setUMask( m ), mode_t( 022 ) );
+    QCOMPARE( Calamares::setUMask( 022 ), m );
+    QCOMPARE( Calamares::setUMask( m ), mode_t( 022 ) );
 }
 
 void
@@ -286,18 +335,18 @@ LibCalamaresTests::testEntropy()
 {
     QByteArray data;
 
-    auto r0 = CalamaresUtils::getEntropy( 0, data );
-    QCOMPARE( CalamaresUtils::EntropySource::None, r0 );
+    auto r0 = Calamares::getEntropy( 0, data );
+    QCOMPARE( Calamares::EntropySource::None, r0 );
     QCOMPARE( data.size(), 0 );
 
-    auto r1 = CalamaresUtils::getEntropy( 16, data );
-    QVERIFY( r1 != CalamaresUtils::EntropySource::None );
+    auto r1 = Calamares::getEntropy( 16, data );
+    QVERIFY( r1 != Calamares::EntropySource::None );
     QCOMPARE( data.size(), 16 );
     // This can randomly fail (but not often)
     QVERIFY( data.at( data.size() - 1 ) != char( 0xcb ) );
 
-    auto r2 = CalamaresUtils::getEntropy( 8, data );
-    QVERIFY( r2 != CalamaresUtils::EntropySource::None );
+    auto r2 = Calamares::getEntropy( 8, data );
+    QVERIFY( r2 != Calamares::EntropySource::None );
     QCOMPARE( data.size(), 8 );
     QCOMPARE( r1, r2 );
     // This can randomly fail (but not often)
@@ -309,12 +358,12 @@ LibCalamaresTests::testPrintableEntropy()
 {
     QString s;
 
-    auto r0 = CalamaresUtils::getPrintableEntropy( 0, s );
-    QCOMPARE( CalamaresUtils::EntropySource::None, r0 );
+    auto r0 = Calamares::getPrintableEntropy( 0, s );
+    QCOMPARE( Calamares::EntropySource::None, r0 );
     QCOMPARE( s.length(), 0 );
 
-    auto r1 = CalamaresUtils::getPrintableEntropy( 16, s );
-    QVERIFY( r1 != CalamaresUtils::EntropySource::None );
+    auto r1 = Calamares::getPrintableEntropy( 16, s );
+    QVERIFY( r1 != Calamares::EntropySource::None );
     QCOMPARE( s.length(), 16 );
     for ( QChar c : s )
     {
@@ -331,14 +380,14 @@ LibCalamaresTests::testOddSizedPrintable()
     QString s;
     for ( int l = 0; l <= 37; ++l )
     {
-        auto r = CalamaresUtils::getPrintableEntropy( l, s );
+        auto r = Calamares::getPrintableEntropy( l, s );
         if ( l == 0 )
         {
-            QCOMPARE( r, CalamaresUtils::EntropySource::None );
+            QCOMPARE( r, Calamares::EntropySource::None );
         }
         else
         {
-            QVERIFY( r != CalamaresUtils::EntropySource::None );
+            QVERIFY( r != Calamares::EntropySource::None );
         }
         QCOMPARE( s.length(), l );
 
@@ -395,7 +444,6 @@ LibCalamaresTests::testPointerSetter()
     QCOMPARE( special, 34 );
 }
 
-
 /* Demonstration of Traits support for has-a-method or not.
  *
  * We have two classes, c1 and c2; one has a method do_the_thing() and the
@@ -436,7 +484,6 @@ public:
     }
 };
 
-
 void
 LibCalamaresTests::testTraits()
 {
@@ -459,7 +506,7 @@ LibCalamaresTests::testTraits()
 void
 LibCalamaresTests::testVariantStringListCode()
 {
-    using namespace CalamaresUtils;
+    using namespace Calamares;
     const QString key( "strings" );
     {
         // Things that are not stringlists
@@ -499,21 +546,21 @@ LibCalamaresTests::testVariantStringListCode()
 void
 LibCalamaresTests::testVariantStringListYAMLDashed()
 {
-    using namespace CalamaresUtils;
+    using namespace Calamares;
     const QString key( "strings" );
 
     // Looks like a stringlist to me
     QTemporaryFile f;
     QVERIFY( f.open() );
     f.write( R"(---
-strings:
-    - aap
-    - noot
-    - mies
-)" );
+             strings:
+             - aap
+             - noot
+             - mies
+           )" );
     f.close();
     bool ok = false;
-    QVariantMap m = loadYaml( f.fileName(), &ok );
+    QVariantMap m = Calamares::YAML::load( f.fileName(), &ok );
 
     QVERIFY( ok );
     QCOMPARE( m.count(), 1 );
@@ -527,18 +574,18 @@ strings:
 void
 LibCalamaresTests::testVariantStringListYAMLBracketed()
 {
-    using namespace CalamaresUtils;
+    using namespace Calamares;
     const QString key( "strings" );
 
     // Looks like a stringlist to me
     QTemporaryFile f;
     QVERIFY( f.open() );
     f.write( R"(---
-strings: [ aap, noot, mies ]
-)" );
+             strings: [ aap, noot, mies ]
+           )" );
     f.close();
     bool ok = false;
-    QVariantMap m = loadYaml( f.fileName(), &ok );
+    QVariantMap m = Calamares::YAML::load( f.fileName(), &ok );
 
     QVERIFY( ok );
     QCOMPARE( m.count(), 1 );
@@ -554,14 +601,14 @@ LibCalamaresTests::testStringTruncation()
 {
     Logger::setupLogLevel( Logger::LOGDEBUG );
 
-    using namespace CalamaresUtils;
-
+    using namespace Calamares::String;
+    // *INDENT-OFF*
     const QString longString( R"(---
 --- src/libcalamares/utils/String.h
 +++ src/libcalamares/utils/String.h
 @@ -62,15 +62,22 @@ DLLEXPORT QString removeDiacritics( const QString& string );
-  */
- DLLEXPORT QString obscure( const QString& string );
+*/
+DLLEXPORT QString obscure( const QString& string );
 
 +/** @brief Parameter for counting lines at beginning and end of string
 + *
@@ -575,6 +622,7 @@ LibCalamaresTests::testStringTruncation()
 +    int atStart = 0;
 +    int atEnd = 0;
 )" );
+    // *INDENT-ON*
 
     const int sufficientLength = 812;
     // There's 18 lines in all
@@ -635,11 +683,14 @@ LibCalamaresTests::testStringTruncationShorter()
 {
     Logger::setupLogLevel( Logger::LOGDEBUG );
 
-    using namespace CalamaresUtils;
+    using namespace Calamares::String;
 
+    // *INDENT-OFF*
     const QString longString( R"(Some strange string artifacts appeared, leading to `{1?}` being
 displayed in various user-facing messages. These have been removed
 and the translations updated.)" );
+    // *INDENT-ON*
+
     const char NEWLINE = '\n';
 
     const int insufficientLength = 42;
@@ -730,7 +781,7 @@ LibCalamaresTests::testStringTruncationDegenerate()
 {
     Logger::setupLogLevel( Logger::LOGDEBUG );
 
-    using namespace CalamaresUtils;
+    using namespace Calamares::String;
 
     // This is quite long, 1 line only, with no newlines
     const QString longString( "The portscout new distfile checker has detected that one or more of your "
@@ -783,7 +834,7 @@ LibCalamaresTests::testStringRemoveLeading()
     QFETCH( QString, result );
 
     const QString initial = string;
-    CalamaresUtils::removeLeading( string, c );
+    Calamares::String::removeLeading( string, c );
     QCOMPARE( string, result );
 }
 
@@ -813,10 +864,64 @@ LibCalamaresTests::testStringRemoveTrailing()
     QFETCH( QString, result );
 
     const QString initial = string;
-    CalamaresUtils::removeTrailing( string, c );
+    Calamares::String::removeTrailing( string, c );
     QCOMPARE( string, result );
 }
 
+void
+LibCalamaresTests::testStringMacroExpander_data()
+{
+    QTest::addColumn< QString >( "source" );
+    QTest::addColumn< QString >( "result" );
+    QTest::addColumn< QStringList >( "errors" );
+
+    QTest::newRow( "empty   " ) << QString() << QString() << QStringList {};
+    QTest::newRow( "constant" ) << QStringLiteral( "bunnies!" ) << QStringLiteral( "bunnies!" ) << QStringList {};
+    QTest::newRow( "escaped " ) << QStringLiteral( "$$bun" ) << QStringLiteral( "$bun" )
+                                << QStringList {};  // Double $$ is an escaped $
+    QTest::newRow( "whole   " ) << QStringLiteral( "${ROOT}" ) << QStringLiteral( "wortel" ) << QStringList {};
+    QTest::newRow( "unbraced" ) << QStringLiteral( "$ROOT" ) << QStringLiteral( "wortel" )
+                                << QStringList {};  // Does not need {}
+    QTest::newRow( "bad-var1" ) << QStringLiteral( "${ROOF}" ) << QStringLiteral( "${ROOF}" )
+                                << QStringList { QStringLiteral( "ROOF" ) };  // Not replaced
+    QTest::newRow( "twice   " ) << QStringLiteral( "${ROOT}x${ROOT}" ) << QStringLiteral( "wortelxwortel" )
+                                << QStringList {};
+    QTest::newRow( "bad-var2" ) << QStringLiteral( "${ROOT}x${ROPE}" ) << QStringLiteral( "wortelx${ROPE}" )
+                                << QStringList { QStringLiteral( "ROPE" ) };  // Not replaced
+    // This is a borked string with a "nested" variable. The variable-name-
+    // scanner goes from ${ to the next } and tries to match that.
+    QTest::newRow( "confuse1" ) << QStringLiteral( "${RO${ROOT}" ) << QStringLiteral( "${ROwortel" )
+                                << QStringList { "RO${ROOT" };
+    // This one doesn't have a { for the first name to match with
+    QTest::newRow( "confuse2" ) << QStringLiteral( "$RO${ROOT}" ) << QStringLiteral( "$ROwortel" )
+                                << QStringList { "RO" };
+    // Here we see it just doesn't nest
+    QTest::newRow( "confuse3" ) << QStringLiteral( "${RO${ROOT}}" ) << QStringLiteral( "${ROwortel}" )
+                                << QStringList { "RO${ROOT" };
+}
+
+void
+LibCalamaresTests::testStringMacroExpander()
+{
+    QHash< QString, QString > dict;
+    dict.insert( QStringLiteral( "ROOT" ), QStringLiteral( "wortel" ) );
+
+    Calamares::String::DictionaryExpander d;
+    d.insert( QStringLiteral( "ROOT" ), QStringLiteral( "wortel" ) );
+
+    QFETCH( QString, source );
+    QFETCH( QString, result );
+    QFETCH( QStringList, errors );
+
+    QString km_expanded = KMacroExpander::expandMacros( source, dict, '$' );
+    QCOMPARE( km_expanded, result );
+
+    QString de_expanded = d.expand( source );
+    QCOMPARE( de_expanded, result );
+    QCOMPARE( d.errorNames(), errors );
+    QCOMPARE( d.hasErrors(), !errors.isEmpty() );
+}
+
 static QString
 dirname( const QTemporaryDir& d )
 {
@@ -927,24 +1032,24 @@ LibCalamaresTests::testCalculateWorkingDirectory()
     gs->insert( "rootMountPoint", tempRoot.path() );
 
     {
-        auto [ ok, d ] = calculateWorkingDirectory( CalamaresUtils::System::RunLocation::RunInHost, QString() );
+        auto [ ok, d ] = calculateWorkingDirectory( Calamares::System::RunLocation::RunInHost, QString() );
         QVERIFY( ok );
         QCOMPARE( d, QDir::current() );
     }
     {
-        auto [ ok, d ] = calculateWorkingDirectory( CalamaresUtils::System::RunLocation::RunInTarget, QString() );
+        auto [ ok, d ] = calculateWorkingDirectory( Calamares::System::RunLocation::RunInTarget, QString() );
         QVERIFY( ok );
         QCOMPARE( d.absolutePath(), tempRoot.path() );
     }
 
     gs->remove( "rootMountPoint" );
     {
-        auto [ ok, d ] = calculateWorkingDirectory( CalamaresUtils::System::RunLocation::RunInHost, QString() );
+        auto [ ok, d ] = calculateWorkingDirectory( Calamares::System::RunLocation::RunInHost, QString() );
         QVERIFY( ok );
         QCOMPARE( d, QDir::current() );
     }
     {
-        auto [ ok, d ] = calculateWorkingDirectory( CalamaresUtils::System::RunLocation::RunInTarget, QString() );
+        auto [ ok, d ] = calculateWorkingDirectory( Calamares::System::RunLocation::RunInTarget, QString() );
         QVERIFY( !ok );
         QCOMPARE( d, QDir::current() );
     }
@@ -993,7 +1098,7 @@ LibCalamaresTests::testRunnerOutput()
 
         Calamares::Utils::Runner r( { "cat" } );
         r.enableOutputProcessing().setInput( QStringLiteral( "hello\nworld\n\n!\n" ) );
-        QObject::connect( &r, &decltype( r )::output, [&collectedOutput]( QString s ) { collectedOutput << s; } );
+        QObject::connect( &r, &decltype( r )::output, [ &collectedOutput ]( QString s ) { collectedOutput << s; } );
 
         {
             auto result = r.run();
@@ -1014,14 +1119,13 @@ LibCalamaresTests::testRunnerOutput()
     }
 }
 
-
-CalamaresUtils::System*
+Calamares::System*
 file_setup( const QTemporaryDir& tempRoot )
 {
-    CalamaresUtils::System* ss = CalamaresUtils::System::instance();
+    Calamares::System* ss = Calamares::System::instance();
     if ( !ss )
     {
-        ss = new CalamaresUtils::System( true );
+        ss = new Calamares::System( true );
     }
 
     Calamares::GlobalStorage* gs
@@ -1050,7 +1154,7 @@ LibCalamaresTests::testReadWriteFile()
 
     QVERIFY( ss );
     {
-        auto fullPath = ss->createTargetFile( "test0", QByteArray(), CalamaresUtils::System::WriteMode::Overwrite );
+        auto fullPath = ss->createTargetFile( "test0", QByteArray(), Calamares::System::WriteMode::Overwrite );
         QVERIFY( fullPath );
         QVERIFY( !fullPath.path().isEmpty() );
 
@@ -1073,7 +1177,7 @@ LibCalamaresTests::testReadWriteFile()
     }
     // But it will if you say so explicitly
     {
-        auto fullPath = ss->createTargetFile( "test0", otherContents, CalamaresUtils::System::WriteMode::Overwrite );
+        auto fullPath = ss->createTargetFile( "test0", otherContents, Calamares::System::WriteMode::Overwrite );
         QVERIFY( fullPath );
         QVERIFY( !fullPath.path().isEmpty() );
 
@@ -1093,7 +1197,6 @@ LibCalamaresTests::testReadWriteFile()
     }
 }
 
-
 QTEST_GUILESS_MAIN( LibCalamaresTests )
 
 #include "utils/moc-warnings.h"
