@@ -9,14 +9,18 @@
 
 #include "InteractiveTerminalPage.h"
 
-#include "utils/CalamaresUtilsGui.h"
+#include "utils/Gui.h"
 #include "utils/Logger.h"
 #include "utils/Retranslator.h"
 #include "viewpages/ViewStep.h"
 #include "widgets/TranslationFix.h"
 
 #include <KParts/ReadOnlyPart>
+#if QT_VERSION < QT_VERSION_CHECK( 6, 0, 0 )
 #include <KParts/kde_terminal_interface.h>
+#else
+#include <kde_terminal_interface.h>
+#endif
 #include <KService>
 #include <kcoreaddons_version.h>
 
@@ -26,7 +30,6 @@
 #include <QMessageBox>
 #include <QVBoxLayout>
 
-
 InteractiveTerminalPage::InteractiveTerminalPage( QWidget* parent )
     : QWidget( parent )
     , m_layout( new QVBoxLayout( this ) )
@@ -43,8 +46,8 @@ void
 InteractiveTerminalPage::errorKonsoleNotInstalled()
 {
     QMessageBox mb( QMessageBox::Critical,
-                    tr( "Konsole not installed" ),
-                    tr( "Please install KDE Konsole and try again!" ),
+                    tr( "Konsole not installed.", "@error" ),
+                    tr( "Please install KDE Konsole and try again!", "@info" ),
                     QMessageBox::Ok );
     Calamares::fixButtonLabels( &mb );
     mb.exec();
@@ -58,10 +61,12 @@ InteractiveTerminalPage::onActivate()
         return;
     }
 
-#if KCOREADDONS_VERSION_MAJOR != 5
-#error Incompatible with not-KF5
-#endif
-#if KCOREADDONS_VERSION_MINOR >= 86
+#if KCOREADDONS_VERSION_MAJOR > 5 || KCOREADDONS_VERSION_MINOR > 200
+#warning Using KF6
+    errorKonsoleNotInstalled();
+    return;
+    KParts::ReadOnlyPart* p = nullptr;
+#elif KCOREADDONS_VERSION_MINOR >= 86
     // 5.86 deprecated a bunch of KService and PluginFactory and related methods
     auto md = KPluginMetaData::findPluginById( QString(), "konsolepart" );
     if ( !md.isValid() )
@@ -109,11 +114,10 @@ InteractiveTerminalPage::onActivate()
     t->sendInput( QString( "%1\n" ).arg( m_command ) );
 }
 
-
 void
 InteractiveTerminalPage::setCommand( const QString& command )
 {
     m_command = command;
     CALAMARES_RETRANSLATE(
-        m_headerLabel->setText( tr( "Executing script: &nbsp;<code>%1</code>" ).arg( m_command ) ); );
+        m_headerLabel->setText( tr( "Executing script: &nbsp;<code>%1</code>", "@info" ).arg( m_command ) ); );
 }
