@@ -3,6 +3,8 @@
  *   SPDX-FileCopyrightText: 2014-2017 Teo Mrnjavac <teo@kde.org>
  *   SPDX-FileCopyrightText: 2017-2019 Adriaan de Groot <groot@kde.org>
  *   SPDX-FileCopyrightText: 2019 Collabora Ltd
+ *   SPDX-FileCopyrightText: 2021 Anubhav Choudhary <ac.10edu@gmail.com>
+ *   SPDX-FileCopyrightText: 2023 Evan James <dalto@fastmail.com>
  *   SPDX-License-Identifier: GPL-3.0-or-later
  *
  *   Calamares is Free Software: see the License-Identifier above.
@@ -27,7 +29,6 @@
 #include "gui/PartitionBarsView.h"
 #include "gui/PartitionLabelsView.h"
 #include "gui/PartitionSplitterWidget.h"
-#include "gui/ReplaceWidget.h"
 #include "gui/ScanningDialog.h"
 
 #include "Branding.h"
@@ -35,16 +36,14 @@
 #include "JobQueue.h"
 #include "partition/PartitionIterator.h"
 #include "partition/PartitionQuery.h"
-#include "utils/CalamaresUtilsGui.h"
+#include "utils/Gui.h"
 #include "utils/Logger.h"
 #include "utils/Retranslator.h"
 #include "utils/Units.h"
 #include "widgets/PrettyRadioButton.h"
 
 #include <kpmcore/core/device.h>
-#ifdef WITH_KPMCORE4API
 #include <kpmcore/core/softwareraid.h>
-#endif
 
 #include <QBoxLayout>
 #include <QButtonGroup>
@@ -55,10 +54,10 @@
 #include <QListView>
 #include <QtConcurrent/QtConcurrent>
 
-using Calamares::PrettyRadioButton;
-using CalamaresUtils::Partition::findPartitionByPath;
-using CalamaresUtils::Partition::isPartitionFreeSpace;
-using CalamaresUtils::Partition::PartitionIterator;
+using Calamares::Partition::findPartitionByPath;
+using Calamares::Partition::isPartitionFreeSpace;
+using Calamares::Partition::PartitionIterator;
+using Calamares::Widgets::PrettyRadioButton;
 using InstallChoice = Config::InstallChoice;
 using SwapChoice = Config::SwapChoice;
 
@@ -97,7 +96,7 @@ ChoicePage::ChoicePage( Config* config, QWidget* parent )
 
     BootInfoWidget* bootInfoWidget = new BootInfoWidget( this );
     m_drivesLayout->insertWidget( 0, bootInfoWidget );
-    m_drivesLayout->insertSpacing( 1, CalamaresUtils::defaultFontHeight() / 2 );
+    m_drivesLayout->insertSpacing( 1, Calamares::defaultFontHeight() / 2 );
 
     m_drivesCombo = new QComboBox( this );
     m_mainLayout->setStretchFactor( m_drivesLayout, 0 );
@@ -113,7 +112,7 @@ ChoicePage::ChoicePage( Config* config, QWidget* parent )
     m_messageLabel->setWordWrap( true );
     m_messageLabel->hide();
 
-    CalamaresUtils::unmarginLayout( m_itemsLayout );
+    Calamares::unmarginLayout( m_itemsLayout );
 
     // Drive selector + preview
     CALAMARES_RETRANSLATE_SLOT( &ChoicePage::retranslate );
@@ -129,22 +128,20 @@ ChoicePage::ChoicePage( Config* config, QWidget* parent )
     updateNextEnabled();
 }
 
-
 ChoicePage::~ChoicePage() {}
 
 void
 ChoicePage::retranslate()
 {
     retranslateUi( this );
-    m_drivesLabel->setText( tr( "Select storage de&vice:" ) );
-    m_previewBeforeLabel->setText( tr( "Current:" ) );
-    m_previewAfterLabel->setText( tr( "After:" ) );
+    m_drivesLabel->setText( tr( "Select storage de&vice:", "@label" ) );
+    m_previewBeforeLabel->setText( tr( "Current:", "@label" ) );
+    m_previewAfterLabel->setText( tr( "After:", "@label" ) );
 
     updateSwapChoicesTr();
     updateChoiceButtonsTr();
 }
 
-
 /** @brief Sets the @p model for the given @p box and adjusts UI sizes to match.
  *
  * The model provides data for drawing the items in the model; the
@@ -176,7 +173,6 @@ ChoicePage::init( PartitionCoreModule* core )
 
     setupChoices();
 
-
     // We need to do this because a PCM revert invalidates the deviceModel.
     connect( core,
              &PartitionCoreModule::reverted,
@@ -196,7 +192,6 @@ ChoicePage::init( PartitionCoreModule* core )
     ChoicePage::applyDeviceChoice();
 }
 
-
 /** @brief Creates a combobox with the given choices in it.
  *
  * Pre-selects the choice given by @p dflt.
@@ -211,10 +206,12 @@ createCombo( const QSet< SwapChoice >& s, SwapChoice dflt )
                            SwapChoice::FullSwap,
                            SwapChoice::ReuseSwap,
                            SwapChoice::SwapFile } )
+    {
         if ( s.contains( c ) )
         {
             box->addItem( QString(), c );
         }
+    }
 
     int dfltIndex = box->findData( dflt );
     if ( dfltIndex >= 0 )
@@ -252,26 +249,25 @@ ChoicePage::setupChoices()
     //  3) Manual
     //  TBD: upgrade option?
 
-    QSize iconSize( CalamaresUtils::defaultIconSize().width() * 2, CalamaresUtils::defaultIconSize().height() * 2 );
+    QSize iconSize( Calamares::defaultIconSize().width() * 2, Calamares::defaultIconSize().height() * 2 );
     m_grp = new QButtonGroup( this );
 
     m_alongsideButton = new PrettyRadioButton;
     m_alongsideButton->setIconSize( iconSize );
     m_alongsideButton->setIcon(
-        CalamaresUtils::defaultPixmap( CalamaresUtils::PartitionAlongside, CalamaresUtils::Original, iconSize ) );
+        Calamares::defaultPixmap( Calamares::PartitionAlongside, Calamares::Original, iconSize ) );
     m_alongsideButton->addToGroup( m_grp, InstallChoice::Alongside );
 
     m_eraseButton = new PrettyRadioButton;
     m_eraseButton->setIconSize( iconSize );
-    m_eraseButton->setIcon(
-        CalamaresUtils::defaultPixmap( CalamaresUtils::PartitionEraseAuto, CalamaresUtils::Original, iconSize ) );
+    m_eraseButton->setIcon( Calamares::defaultPixmap( Calamares::PartitionEraseAuto, Calamares::Original, iconSize ) );
     m_eraseButton->addToGroup( m_grp, InstallChoice::Erase );
 
     m_replaceButton = new PrettyRadioButton;
 
     m_replaceButton->setIconSize( iconSize );
     m_replaceButton->setIcon(
-        CalamaresUtils::defaultPixmap( CalamaresUtils::PartitionReplaceOs, CalamaresUtils::Original, iconSize ) );
+        Calamares::defaultPixmap( Calamares::PartitionReplaceOs, Calamares::Original, iconSize ) );
     m_replaceButton->addToGroup( m_grp, InstallChoice::Replace );
 
     // Fill up swap options
@@ -289,6 +285,16 @@ ChoicePage::setupChoices()
             m_eraseFsTypesChoiceComboBox, &QComboBox::currentTextChanged, m_config, &Config::setEraseFsTypeChoice );
         connect( m_config, &Config::eraseModeFilesystemChanged, this, &ChoicePage::onActionChanged );
         m_eraseButton->addOptionsComboBox( m_eraseFsTypesChoiceComboBox );
+
+        // Also offer it for "replace
+        m_replaceFsTypesChoiceComboBox = new QComboBox;
+        m_replaceFsTypesChoiceComboBox->addItems( m_config->eraseFsTypes() );
+        connect( m_replaceFsTypesChoiceComboBox,
+                 &QComboBox::currentTextChanged,
+                 m_config,
+                 &Config::setReplaceFilesystemChoice );
+        connect( m_config, &Config::replaceModeFilesystemChanged, this, &ChoicePage::onActionChanged );
+        m_replaceButton->addOptionsComboBox( m_replaceFsTypesChoiceComboBox );
     }
 
     m_itemsLayout->addWidget( m_alongsideButton );
@@ -298,19 +304,14 @@ ChoicePage::setupChoices()
     m_somethingElseButton = new PrettyRadioButton;
     m_somethingElseButton->setIconSize( iconSize );
     m_somethingElseButton->setIcon(
-        CalamaresUtils::defaultPixmap( CalamaresUtils::PartitionManual, CalamaresUtils::Original, iconSize ) );
+        Calamares::defaultPixmap( Calamares::PartitionManual, Calamares::Original, iconSize ) );
     m_itemsLayout->addWidget( m_somethingElseButton );
     m_somethingElseButton->addToGroup( m_grp, InstallChoice::Manual );
 
     m_itemsLayout->addStretch();
 
-#if ( QT_VERSION < QT_VERSION_CHECK( 5, 15, 0 ) )
-    auto buttonSignal = QOverload< int, bool >::of( &QButtonGroup::buttonToggled );
-#else
-    auto buttonSignal = &QButtonGroup::idToggled;
-#endif
     connect( m_grp,
-             buttonSignal,
+             &QButtonGroup::idToggled,
              this,
              [ this ]( int id, bool checked )
              {
@@ -351,7 +352,6 @@ ChoicePage::setupChoices()
     updateChoiceButtonsTr();
 }
 
-
 /**
  * @brief ChoicePage::selectedDevice queries the device picker (which may be a combo or
  *      a list view) to get a pointer to the currently selected Device.
@@ -368,7 +368,6 @@ ChoicePage::selectedDevice()
     return currentDevice;
 }
 
-
 void
 ChoicePage::hideButtons()
 {
@@ -391,7 +390,6 @@ ChoicePage::checkInstallChoiceRadioButton( InstallChoice c )
     m_grp->setExclusive( true );
 }
 
-
 /**
  * @brief ChoicePage::applyDeviceChoice handler for the selected event of the device
  *      picker. Calls ChoicePage::selectedDevice() to get the current Device*, then
@@ -426,7 +424,6 @@ ChoicePage::applyDeviceChoice()
     }
 }
 
-
 void
 ChoicePage::continueApplyDeviceChoice()
 {
@@ -450,7 +447,6 @@ ChoicePage::continueApplyDeviceChoice()
     if ( m_lastSelectedDeviceIndex != m_drivesCombo->currentIndex() )
     {
         m_lastSelectedDeviceIndex = m_drivesCombo->currentIndex();
-        m_lastSelectedActionIndex = -1;
         m_config->setInstallChoice( m_config->initialInstallChoice() );
         checkInstallChoiceRadioButton( m_config->installChoice() );
     }
@@ -462,24 +458,24 @@ ChoicePage::continueApplyDeviceChoice()
 void
 ChoicePage::onActionChanged()
 {
+    if ( m_enableEncryptionWidget )
+    {
+        if ( m_config->installChoice() == InstallChoice::Erase && m_eraseFsTypesChoiceComboBox )
+        {
+            m_encryptWidget->setFilesystem( FileSystem::typeForName( m_eraseFsTypesChoiceComboBox->currentText() ) );
+        }
+        else if ( m_config->installChoice() == InstallChoice::Replace && m_replaceFsTypesChoiceComboBox )
+        {
+            m_encryptWidget->setFilesystem( FileSystem::typeForName( m_replaceFsTypesChoiceComboBox->currentText() ) );
+        }
+    }
+
     Device* currd = selectedDevice();
     if ( currd )
     {
         applyActionChoice( m_config->installChoice() );
     }
 
-    // Whole disk encryption isn't implemented for zfs so disable the option for now
-    if ( m_eraseFsTypesChoiceComboBox != nullptr && m_enableEncryptionWidget )
-    {
-        if ( m_eraseFsTypesChoiceComboBox->currentText() == "zfs" )
-        {
-            m_encryptWidget->hide();
-        }
-        else
-        {
-            m_encryptWidget->show();
-        }
-    }
     updateNextEnabled();
 }
 
@@ -496,9 +492,9 @@ ChoicePage::onEraseSwapChoiceChanged()
 void
 ChoicePage::applyActionChoice( InstallChoice choice )
 {
-    cDebug() << "Prev" << m_lastSelectedActionIndex << "InstallChoice" << choice
-             << Config::installChoiceNames().find( choice );
+    cDebug() << "InstallChoice" << choice << Config::installChoiceNames().find( choice );
     m_beforePartitionBarsView->selectionModel()->disconnect( SIGNAL( currentRowChanged( QModelIndex, QModelIndex ) ) );
+    auto priorSelection = m_beforePartitionBarsView->selectionModel()->currentIndex();
     m_beforePartitionBarsView->selectionModel()->clearSelection();
     m_beforePartitionBarsView->selectionModel()->clearCurrentIndex();
 
@@ -509,9 +505,10 @@ ChoicePage::applyActionChoice( InstallChoice choice )
         auto gs = Calamares::JobQueue::instance()->globalStorage();
         PartitionActions::Choices::AutoPartitionOptions options { gs->value( "defaultPartitionTableType" ).toString(),
                                                                   m_config->eraseFsType(),
+                                                                  m_config->luksFileSystemType(),
                                                                   m_encryptWidget->passphrase(),
                                                                   gs->value( "efiSystemPartition" ).toString(),
-                                                                  CalamaresUtils::GiBtoBytes(
+                                                                  Calamares::GiBtoBytes(
                                                                       gs->value( "requiredStorageGiB" ).toDouble() ),
                                                                   m_config->swapChoice() };
 
@@ -556,6 +553,12 @@ ChoicePage::applyActionChoice( InstallChoice choice )
                  this,
                  SLOT( onPartitionToReplaceSelected( QModelIndex, QModelIndex ) ),
                  Qt::UniqueConnection );
+
+        // Maintain the selection for replace
+        if ( priorSelection.isValid() )
+        {
+            m_beforePartitionBarsView->selectionModel()->setCurrentIndex( priorSelection, QItemSelectionModel::Select );
+        }
         break;
 
     case InstallChoice::Alongside:
@@ -592,7 +595,6 @@ ChoicePage::applyActionChoice( InstallChoice choice )
     updateActionChoicePreview( choice );
 }
 
-
 void
 ChoicePage::doAlongsideSetupSplitter( const QModelIndex& current, const QModelIndex& previous )
 {
@@ -623,7 +625,7 @@ ChoicePage::doAlongsideSetupSplitter( const QModelIndex& current, const QModelIn
     double requiredStorageGB
         = Calamares::JobQueue::instance()->globalStorage()->value( "requiredStorageGiB" ).toDouble();
 
-    qint64 requiredStorageB = CalamaresUtils::GiBtoBytes( requiredStorageGB + 0.1 + 2.0 );
+    qint64 requiredStorageB = Calamares::GiBtoBytes( requiredStorageGB + 0.1 + 2.0 );
 
     m_afterPartitionSplitterWidget->setSplitPartition( part->partitionPath(),
                                                        qRound64( part->used() * 1.1 ),
@@ -640,7 +642,6 @@ ChoicePage::doAlongsideSetupSplitter( const QModelIndex& current, const QModelIn
     updateNextEnabled();
 }
 
-
 void
 ChoicePage::onEncryptWidgetStateChanged()
 {
@@ -663,7 +664,6 @@ ChoicePage::onEncryptWidgetStateChanged()
     updateNextEnabled();
 }
 
-
 void
 ChoicePage::onHomeCheckBoxStateChanged()
 {
@@ -674,7 +674,6 @@ ChoicePage::onHomeCheckBoxStateChanged()
     }
 }
 
-
 void
 ChoicePage::onLeave()
 {
@@ -735,7 +734,6 @@ ChoicePage::onLeave()
     }
 }
 
-
 void
 ChoicePage::doAlongsideApply()
 {
@@ -765,6 +763,7 @@ ChoicePage::doAlongsideApply()
             m_core->layoutApply( dev,
                                  newLastSector + 2,
                                  oldLastSector,
+                                 m_config->luksFileSystemType(),
                                  m_encryptWidget->passphrase(),
                                  candidate->parent(),
                                  candidate->roles() );
@@ -775,7 +774,6 @@ ChoicePage::doAlongsideApply()
     }
 }
 
-
 void
 ChoicePage::onPartitionToReplaceSelected( const QModelIndex& current, const QModelIndex& previous )
 {
@@ -791,7 +789,6 @@ ChoicePage::onPartitionToReplaceSelected( const QModelIndex& current, const QMod
     doReplaceSelectedPartition( current );
 }
 
-
 void
 ChoicePage::doReplaceSelectedPartition( const QModelIndex& current )
 {
@@ -839,6 +836,7 @@ ChoicePage::doReplaceSelectedPartition( const QModelIndex& current )
                     m_core->layoutApply( selectedDevice(),
                                          selectedPartition->firstSector(),
                                          selectedPartition->lastSector(),
+                                         m_config->luksFileSystemType(),
                                          m_encryptWidget->passphrase(),
                                          newParent,
                                          newRoles );
@@ -855,10 +853,12 @@ ChoicePage::doReplaceSelectedPartition( const QModelIndex& current )
                         // m_reuseHomeCheckBox visible and set its text to something meaningful.
                         homePartitionPath->clear();
                         for ( const OsproberEntry& osproberEntry : m_core->osproberEntries() )
+                        {
                             if ( osproberEntry.path == partPath )
                             {
                                 *homePartitionPath = osproberEntry.homePath;
                             }
+                        }
                         if ( homePartitionPath->isEmpty() )
                         {
                             doReuseHomePartition = false;
@@ -870,7 +870,8 @@ ChoicePage::doReplaceSelectedPartition( const QModelIndex& current )
                                                               selectedDevice(),
                                                               selectedPartition,
                                                               { gs->value( "defaultPartitionType" ).toString(),
-                                                                gs->value( "defaultFileSystemType" ).toString(),
+                                                                m_config->replaceModeFilesystem(),
+                                                                m_config->luksFileSystemType(),
                                                                 m_encryptWidget->passphrase() } );
                         Partition* homePartition = findPartitionByPath( { selectedDevice() }, *homePartitionPath );
 
@@ -891,22 +892,27 @@ ChoicePage::doReplaceSelectedPartition( const QModelIndex& current )
         {
             m_reuseHomeCheckBox->setVisible( !homePartitionPath->isEmpty() );
             if ( !homePartitionPath->isEmpty() )
-                m_reuseHomeCheckBox->setText( tr( "Reuse %1 as home partition for %2." )
+            {
+                m_reuseHomeCheckBox->setText( tr( "Reuse %1 as home partition for %2", "@label" )
                                                   .arg( *homePartitionPath )
                                                   .arg( Calamares::Branding::instance()->shortProductName() ) );
+            }
             delete homePartitionPath;
 
             if ( m_isEfi )
+            {
                 setupEfiSystemPartitionSelector();
+            }
 
             updateNextEnabled();
             if ( !m_bootloaderComboBox.isNull() && m_bootloaderComboBox->currentIndex() < 0 )
+            {
                 m_bootloaderComboBox->setCurrentIndex( m_lastSelectedDeviceIndex );
+            }
         },
         this );
 }
 
-
 /**
  * @brief clear and then rebuild the contents of the preview widget
  *
@@ -934,7 +940,7 @@ ChoicePage::updateDeviceStatePreview()
 
     layout = new QVBoxLayout;
     m_previewBeforeFrame->setLayout( layout );
-    CalamaresUtils::unmarginLayout( layout );
+    Calamares::unmarginLayout( layout );
     layout->setSpacing( 6 );
 
     PartitionBarsView::NestedPartitionsMode mode
@@ -980,7 +986,6 @@ ChoicePage::updateDeviceStatePreview()
     layout->addWidget( m_beforePartitionLabelsView );
 }
 
-
 /**
  * @brief rebuild the contents of the preview for the PCM-proposed state.
  *
@@ -1007,7 +1012,7 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
 
     QVBoxLayout* layout = new QVBoxLayout;
     m_previewAfterFrame->setLayout( layout );
-    CalamaresUtils::unmarginLayout( layout );
+    Calamares::unmarginLayout( layout );
     layout->setSpacing( 6 );
 
     PartitionBarsView::NestedPartitionsMode mode
@@ -1026,7 +1031,7 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
         {
             m_encryptWidget->show();
         }
-        m_previewBeforeLabel->setText( tr( "Current:" ) );
+        m_previewBeforeLabel->setText( tr( "Current:", "@label" ) );
         m_selectLabel->setText( tr( "<strong>Select a partition to shrink, "
                                     "then drag the bottom bar to resize</strong>" ) );
         m_selectLabel->show();
@@ -1038,6 +1043,12 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
         QLabel* sizeLabel = new QLabel( m_previewAfterFrame );
         layout->addWidget( sizeLabel );
         sizeLabel->setWordWrap( true );
+
+        if ( !m_isEfi )
+        {
+            layout->addWidget( createBootloaderPanel() );
+        }
+
         connect( m_afterPartitionSplitterWidget,
                  &PartitionSplitterWidget::partitionResized,
                  this,
@@ -1046,10 +1057,10 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
                      Q_UNUSED( path )
                      sizeLabel->setText(
                          tr( "%1 will be shrunk to %2MiB and a new "
-                             "%3MiB partition will be created for %4." )
+                             "%3MiB partition will be created for %4.", "@info, %1 is partition name, %4 is product name" )
                              .arg( m_beforePartitionBarsView->selectionModel()->currentIndex().data().toString() )
-                             .arg( CalamaresUtils::BytesToMiB( size ) )
-                             .arg( CalamaresUtils::BytesToMiB( sizeNext ) )
+                             .arg( Calamares::BytesToMiB( size ) )
+                             .arg( Calamares::BytesToMiB( sizeNext ) )
                              .arg( Calamares::Branding::instance()->shortProductName() ) );
                  } );
 
@@ -1070,11 +1081,8 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
     case InstallChoice::Erase:
     case InstallChoice::Replace:
     {
-        if ( m_enableEncryptionWidget )
-        {
-            m_encryptWidget->show();
-        }
-        m_previewBeforeLabel->setText( tr( "Current:" ) );
+        m_encryptWidget->setVisible( shouldShowEncryptWidget( choice ) );
+        m_previewBeforeLabel->setText( tr( "Current:", "@label" ) );
         m_afterPartitionBarsView = new PartitionBarsView( m_previewAfterFrame );
         m_afterPartitionBarsView->setNestedPartitionsMode( mode );
         m_afterPartitionLabelsView = new PartitionLabelsView( m_previewAfterFrame );
@@ -1096,51 +1104,7 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
 
         if ( !m_isEfi )
         {
-            QWidget* eraseWidget = new QWidget;
-
-            QHBoxLayout* eraseLayout = new QHBoxLayout;
-            eraseWidget->setLayout( eraseLayout );
-            eraseLayout->setContentsMargins( 0, 0, 0, 0 );
-            QLabel* eraseBootloaderLabel = new QLabel( eraseWidget );
-            eraseLayout->addWidget( eraseBootloaderLabel );
-            eraseBootloaderLabel->setText( tr( "Boot loader location:" ) );
-
-            m_bootloaderComboBox = createBootloaderComboBox( eraseWidget );
-            connect( m_core->bootLoaderModel(),
-                     &QAbstractItemModel::modelReset,
-                     [ this ]()
-                     {
-                         if ( !m_bootloaderComboBox.isNull() )
-                         {
-                             Calamares::restoreSelectedBootLoader( *m_bootloaderComboBox,
-                                                                   m_core->bootLoaderInstallPath() );
-                         }
-                     } );
-            connect(
-                m_core,
-                &PartitionCoreModule::deviceReverted,
-                this,
-                [ this ]( Device* dev )
-                {
-                    Q_UNUSED( dev )
-                    if ( !m_bootloaderComboBox.isNull() )
-                    {
-                        if ( m_bootloaderComboBox->model() != m_core->bootLoaderModel() )
-                        {
-                            m_bootloaderComboBox->setModel( m_core->bootLoaderModel() );
-                        }
-
-                        m_bootloaderComboBox->setCurrentIndex( m_lastSelectedDeviceIndex );
-                    }
-                },
-                Qt::QueuedConnection );
-            // ^ Must be Queued so it's sure to run when the widget is already visible.
-
-            eraseLayout->addWidget( m_bootloaderComboBox );
-            eraseBootloaderLabel->setBuddy( m_bootloaderComboBox );
-            eraseLayout->addStretch();
-
-            layout->addWidget( eraseWidget );
+            layout->addWidget( createBootloaderPanel() );
         }
 
         m_previewAfterFrame->show();
@@ -1162,7 +1126,7 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
             m_beforePartitionLabelsView->setSelectionFilter( filter );
 
             m_selectLabel->show();
-            m_selectLabel->setText( tr( "<strong>Select a partition to install on</strong>" ) );
+            m_selectLabel->setText( tr( "<strong>Select a partition to install on</strong>", "@label" ) );
         }
 
         break;
@@ -1171,7 +1135,7 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
     case InstallChoice::Manual:
         m_selectLabel->hide();
         m_previewAfterFrame->hide();
-        m_previewBeforeLabel->setText( tr( "Current:" ) );
+        m_previewBeforeLabel->setText( tr( "Current:", "@label" ) );
         m_previewAfterLabel->hide();
         m_encryptWidget->hide();
         break;
@@ -1212,7 +1176,6 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
     updateNextEnabled();
 }
 
-
 void
 ChoicePage::setupEfiSystemPartitionSelector()
 {
@@ -1225,21 +1188,21 @@ ChoicePage::setupEfiSystemPartitionSelector()
     {
         m_efiLabel->setText( tr( "An EFI system partition cannot be found anywhere "
                                  "on this system. Please go back and use manual "
-                                 "partitioning to set up %1." )
+                                 "partitioning to set up %1.", "@info, %1 is product name" )
                                  .arg( Calamares::Branding::instance()->shortProductName() ) );
         updateNextEnabled();
     }
     else if ( efiSystemPartitions.count() == 1 )  //probably most usual situation
     {
         m_efiLabel->setText( tr( "The EFI system partition at %1 will be used for "
-                                 "starting %2." )
+                                 "starting %2.", "@info, %1 is partition path, %2 is product name" )
                                  .arg( efiSystemPartitions.first()->partitionPath() )
                                  .arg( Calamares::Branding::instance()->shortProductName() ) );
     }
     else
     {
         m_efiComboBox->show();
-        m_efiLabel->setText( tr( "EFI system partition:" ) );
+        m_efiLabel->setText( tr( "EFI system partition:", "@label" ) );
         for ( int i = 0; i < efiSystemPartitions.count(); ++i )
         {
             Partition* efiPartition = efiSystemPartitions.at( i );
@@ -1254,35 +1217,6 @@ ChoicePage::setupEfiSystemPartitionSelector()
     }
 }
 
-
-QComboBox*
-ChoicePage::createBootloaderComboBox( QWidget* parent )
-{
-    QComboBox* comboForBootloader = new QComboBox( parent );
-    comboForBootloader->setModel( m_core->bootLoaderModel() );
-
-    // When the chosen bootloader device changes, we update the choice in the PCM
-    connect( comboForBootloader,
-             QOverload< int >::of( &QComboBox::currentIndexChanged ),
-             this,
-             [ this ]( int newIndex )
-             {
-                 QComboBox* bootloaderCombo = qobject_cast< QComboBox* >( sender() );
-                 if ( bootloaderCombo )
-                 {
-                     QVariant var = bootloaderCombo->itemData( newIndex, BootLoaderModel::BootLoaderPathRole );
-                     if ( !var.isValid() )
-                     {
-                         return;
-                     }
-                     m_core->setBootLoaderInstallPath( var.toString() );
-                 }
-             } );
-
-    return comboForBootloader;
-}
-
-
 static inline void
 force_uncheck( QButtonGroup* grp, PrettyRadioButton* button )
 {
@@ -1361,14 +1295,12 @@ ChoicePage::setupActions()
     bool isInactiveRAID = false;
     bool matchTableType = false;
 
-#ifdef WITH_KPMCORE4API
     if ( currentDevice->type() == Device::Type::SoftwareRAID_Device
          && static_cast< SoftwareRAID* >( currentDevice )->status() == SoftwareRAID::Status::Inactive )
     {
         cDebug() << Logger::SubEntry << "part of an inactive RAID device";
         isInactiveRAID = true;
     }
-#endif
 
     PartitionTable::TableType tableType = PartitionTable::unknownTableType;
     if ( currentDevice->partitionTable() )
@@ -1445,7 +1377,6 @@ ChoicePage::setupActions()
                                             "This will <font color=\"red\">delete</font> all data "
                                             "currently present on the selected storage device." ) );
 
-
                 m_replaceButton->setText( tr( "<strong>Replace a partition</strong><br/>"
                                               "Replaces a partition with %1." )
                                               .arg( Calamares::Branding::instance()->shortVersionedName() ) ); );
@@ -1574,12 +1505,12 @@ ChoicePage::setupActions()
     {
         if ( atLeastOneIsMounted )
         {
-            m_messageLabel->setText( tr( "This storage device has one of its partitions <strong>mounted</strong>." ) );
+            m_messageLabel->setText( tr( "This storage device has one of its partitions <strong>mounted</strong>.", "@info" ) );
         }
         else
         {
             m_messageLabel->setText(
-                tr( "This storage device is a part of an <strong>inactive RAID</strong> device." ) );
+                tr( "This storage device is a part of an <strong>inactive RAID</strong> device.", "@info" ) );
         }
 
         m_messageLabel->show();
@@ -1589,7 +1520,6 @@ ChoicePage::setupActions()
     }
 }
 
-
 OsproberEntryList
 ChoicePage::getOsproberEntriesForDevice( Device* device ) const
 {
@@ -1604,14 +1534,12 @@ ChoicePage::getOsproberEntriesForDevice( Device* device ) const
     return eList;
 }
 
-
 bool
 ChoicePage::isNextEnabled() const
 {
     return m_nextEnabled;
 }
 
-
 bool
 ChoicePage::calculateNextEnabled() const
 {
@@ -1664,7 +1592,6 @@ ChoicePage::calculateNextEnabled() const
     return true;
 }
 
-
 void
 ChoicePage::updateNextEnabled()
 {
@@ -1698,7 +1625,7 @@ ChoicePage::updateSwapChoicesTr()
             // toInt() returns 0 on failure, so check for ok
             if ( ok )  // It was explicitly set to 0
             {
-                m_eraseSwapChoiceComboBox->setItemText( index, tr( "No Swap" ) );
+                m_eraseSwapChoiceComboBox->setItemText( index, tr( "No swap", "@label" ) );
             }
             else
             {
@@ -1707,16 +1634,16 @@ ChoicePage::updateSwapChoicesTr()
             }
             break;
         case SwapChoice::ReuseSwap:
-            m_eraseSwapChoiceComboBox->setItemText( index, tr( "Reuse Swap" ) );
+            m_eraseSwapChoiceComboBox->setItemText( index, tr( "Reuse swap", "@label" ) );
             break;
         case SwapChoice::SmallSwap:
-            m_eraseSwapChoiceComboBox->setItemText( index, tr( "Swap (no Hibernate)" ) );
+            m_eraseSwapChoiceComboBox->setItemText( index, tr( "Swap (no Hibernate)", "@label" ) );
             break;
         case SwapChoice::FullSwap:
-            m_eraseSwapChoiceComboBox->setItemText( index, tr( "Swap (with Hibernate)" ) );
+            m_eraseSwapChoiceComboBox->setItemText( index, tr( "Swap (with Hibernate)", "@label" ) );
             break;
         case SwapChoice::SwapFile:
-            m_eraseSwapChoiceComboBox->setItemText( index, tr( "Swap to file" ) );
+            m_eraseSwapChoiceComboBox->setItemText( index, tr( "Swap to file", "@label" ) );
             break;
         default:
             cWarning() << "Box item" << index << m_eraseSwapChoiceComboBox->itemText( index ) << "has role" << value;
@@ -1746,3 +1673,90 @@ ChoicePage::setLastSelectedDeviceIndex( int index )
     m_lastSelectedDeviceIndex = index;
     m_drivesCombo->setCurrentIndex( m_lastSelectedDeviceIndex );
 }
+
+QWidget*
+ChoicePage::createBootloaderPanel()
+{
+    QWidget* panelWidget = new QWidget;
+
+    QHBoxLayout* mainLayout = new QHBoxLayout;
+    panelWidget->setLayout( mainLayout );
+    mainLayout->setContentsMargins( 0, 0, 0, 0 );
+    QLabel* widgetLabel = new QLabel( panelWidget );
+    mainLayout->addWidget( widgetLabel );
+    widgetLabel->setText( tr( "Bootloader location:", "@label" ) );
+
+    QComboBox* comboForBootloader = new QComboBox( panelWidget );
+    comboForBootloader->setModel( m_core->bootLoaderModel() );
+
+    // When the chosen bootloader device changes, we update the choice in the PCM
+    connect( comboForBootloader,
+             QOverload< int >::of( &QComboBox::currentIndexChanged ),
+             this,
+             [ this ]( int newIndex )
+             {
+                 QComboBox* bootloaderCombo = qobject_cast< QComboBox* >( sender() );
+                 if ( bootloaderCombo )
+                 {
+                     QVariant var = bootloaderCombo->itemData( newIndex, BootLoaderModel::BootLoaderPathRole );
+                     if ( !var.isValid() )
+                     {
+                         return;
+                     }
+                     m_core->setBootLoaderInstallPath( var.toString() );
+                 }
+             } );
+    m_bootloaderComboBox = comboForBootloader;
+
+    connect( m_core->bootLoaderModel(),
+             &QAbstractItemModel::modelReset,
+             [ this ]()
+             {
+                 if ( !m_bootloaderComboBox.isNull() )
+                 {
+                     Calamares::restoreSelectedBootLoader( *m_bootloaderComboBox, m_core->bootLoaderInstallPath() );
+                 }
+             } );
+    connect(
+        m_core,
+        &PartitionCoreModule::deviceReverted,
+        this,
+        [ this ]( Device* )
+        {
+            if ( !m_bootloaderComboBox.isNull() )
+            {
+                if ( m_bootloaderComboBox->model() != m_core->bootLoaderModel() )
+                {
+                    m_bootloaderComboBox->setModel( m_core->bootLoaderModel() );
+                }
+
+                m_bootloaderComboBox->setCurrentIndex( m_lastSelectedDeviceIndex );
+            }
+        },
+        Qt::QueuedConnection );
+    // ^ Must be Queued so it's sure to run when the widget is already visible.
+
+    mainLayout->addWidget( m_bootloaderComboBox );
+    widgetLabel->setBuddy( m_bootloaderComboBox );
+    mainLayout->addStretch();
+
+    return panelWidget;
+}
+
+bool
+ChoicePage::shouldShowEncryptWidget( Config::InstallChoice choice ) const
+{
+    bool suitableFS = true;
+    if ( !m_config->allowZfsEncryption()
+         && ( ( m_eraseFsTypesChoiceComboBox && m_eraseFsTypesChoiceComboBox->isVisible()
+                && m_eraseFsTypesChoiceComboBox->currentText() == "zfs" )
+              || ( m_replaceFsTypesChoiceComboBox && m_replaceFsTypesChoiceComboBox->isVisible()
+                   && m_replaceFsTypesChoiceComboBox->currentText() == "zfs" ) ) )
+    {
+        suitableFS = false;
+    }
+
+    const bool suitableChoice
+        = choice == InstallChoice::Erase || choice == InstallChoice::Alongside || choice == InstallChoice::Replace;
+    return suitableChoice && m_enableEncryptionWidget && suitableFS;
+}
