@@ -16,16 +16,16 @@
 
 #include "GlobalStorage.h"
 #include "JobQueue.h"
+#include "compat/Variant.h"
 #include "utils/Logger.h"
 #include "utils/String.h"
+#include "utils/StringExpander.h"
 #include "utils/Variant.h"
 
-#include <KMacroExpander>
-
 #include <QCoreApplication>
 #include <QFile>
 #include <QMetaProperty>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QTimer>
 
 #ifdef HAVE_ICU
@@ -42,10 +42,10 @@ static const char TRANSLITERATOR_ID[] = "Russian-Latin/BGN;"
 
 #include <memory>
 
-static const QRegExp USERNAME_RX( "^[a-z_][a-z0-9_-]*[$]?$" );
+static const QRegularExpression USERNAME_RX( "^[a-z_][a-z0-9_-]*[$]?$" );  // Note anchors begin and end
 static constexpr const int USERNAME_MAX_LENGTH = 31;
 
-static const QRegExp HOSTNAME_RX( "^[a-zA-Z0-9][-a-zA-Z0-9_]*$" );
+static const QRegularExpression HOSTNAME_RX( "^[a-zA-Z0-9][-a-zA-Z0-9_]*$" );  // Note anchors begin and end
 static constexpr const int HOSTNAME_MIN_LENGTH = 2;
 static constexpr const int HOSTNAME_MAX_LENGTH = 63;
 
@@ -78,6 +78,20 @@ updateGSAutoLogin( bool doAutoLogin, const QString& login )
     }
 }
 
+static const QStringList&
+alwaysForbiddenLoginNames()
+{
+    static QStringList s { QStringLiteral( "root" ), QStringLiteral( "nobody" ) };
+    return s;
+}
+
+static const QStringList&
+alwaysForbiddenHostNames()
+{
+    static QStringList s { QStringLiteral( "localhost" ) };
+    return s;
+}
+
 const NamedEnumTable< HostNameAction >&
 hostnameActionNames()
 {
@@ -98,6 +112,8 @@ hostnameActionNames()
 
 Config::Config( QObject* parent )
     : Calamares::ModuleSystem::Config( parent )
+    , m_forbiddenHostNames( alwaysForbiddenHostNames() )
+    , m_forbiddenLoginNames( alwaysForbiddenLoginNames() )
 {
     emit readyChanged( m_isReady );  // false
 
@@ -185,7 +201,6 @@ Config::setSudoersGroup( const QString& group )
     }
 }
 
-
 void
 Config::setLoginName( const QString& login )
 {
@@ -201,10 +216,9 @@ Config::setLoginName( const QString& login )
 }
 
 const QStringList&
-Config::forbiddenLoginNames()
+Config::forbiddenLoginNames() const
 {
-    static QStringList forbidden { "root" };
-    return forbidden;
+    return m_forbiddenLoginNames;
 }
 
 QString
@@ -220,24 +234,23 @@ Config::loginNameStatus() const
     {
         return tr( "Your username is too long." );
     }
-    for ( const QString& badName : forbiddenLoginNames() )
-    {
-        if ( 0 == QString::compare( badName, m_loginName, Qt::CaseSensitive ) )
-        {
-            return tr( "'%1' is not allowed as username." ).arg( badName );
-        }
-    }
 
-    QRegExp validateFirstLetter( "^[a-z_]" );
-    if ( validateFirstLetter.indexIn( m_loginName ) != 0 )
+    QRegularExpression validateFirstLetter( "^[a-z_]" );
+    if ( m_loginName.indexOf( validateFirstLetter ) != 0 )
     {
         return tr( "Your username must start with a lowercase letter or underscore." );
     }
-    if ( !USERNAME_RX.exactMatch( m_loginName ) )
+    if ( m_loginName.indexOf( USERNAME_RX ) != 0 )
     {
         return tr( "Only lowercase letters, numbers, underscore and hyphen are allowed." );
     }
 
+    // Although we've made the list lower-case, and the RE above forces lower-case, still pass the flag
+    if ( forbiddenLoginNames().contains( m_loginName, Qt::CaseInsensitive ) )
+    {
+        return tr( "'%1' is not allowed as username." ).arg( m_loginName );
+    }
+
     return QString();
 }
 
@@ -268,10 +281,9 @@ Config::setHostName( const QString& host )
 }
 
 const QStringList&
-Config::forbiddenHostNames()
+Config::forbiddenHostNames() const
 {
-    static QStringList forbidden { "localhost" };
-    return forbidden;
+    return m_forbiddenHostNames;
 }
 
 QString
@@ -291,15 +303,14 @@ Config::hostnameStatus() const
     {
         return tr( "Your hostname is too long." );
     }
-    for ( const QString& badName : forbiddenHostNames() )
+
+    // "LocalHost" is just as forbidden as "localhost"
+    if ( forbiddenHostNames().contains( m_hostname, Qt::CaseInsensitive ) )
     {
-        if ( 0 == QString::compare( badName, m_hostname, Qt::CaseSensitive ) )
-        {
-            return tr( "'%1' is not allowed as hostname." ).arg( badName );
-        }
+        return tr( "'%1' is not allowed as hostname." ).arg( m_hostname );
     }
 
-    if ( !HOSTNAME_RX.exactMatch( m_hostname ) )
+    if ( m_hostname.indexOf( HOSTNAME_RX ) != 0 )
     {
         return tr( "Only letters, numbers, underscore and hyphen are allowed." );
     }
@@ -310,7 +321,7 @@ Config::hostnameStatus() const
 static QString
 cleanupForHostname( const QString& s )
 {
-    QRegExp dmirx( "[^a-zA-Z0-9]", Qt::CaseInsensitive );
+    QRegularExpression dmirx( "(^Apple|\\(.*\\)|[^a-zA-Z0-9])", QRegularExpression::CaseInsensitiveOption );
     return s.toLower().replace( dmirx, " " ).remove( ' ' );
 }
 
@@ -328,15 +339,30 @@ guessProductName()
     if ( !tried )
     {
         QFile dmiFile( QStringLiteral( "/sys/devices/virtual/dmi/id/product_name" ) );
+        QFile modelFile( QStringLiteral( "/proc/device-tree/model" ) );
 
         if ( dmiFile.exists() && dmiFile.open( QIODevice::ReadOnly ) )
         {
             dmiProduct = cleanupForHostname( QString::fromLocal8Bit( dmiFile.readAll().simplified().data() ) );
+            if ( !dmiProduct.isEmpty() )
+            {
+                tried = true;
+                return dmiProduct;
+            }
         }
-        if ( dmiProduct.isEmpty() )
+
+        if ( modelFile.exists() && modelFile.open( QIODevice::ReadOnly ) )
         {
-            dmiProduct = QStringLiteral( "pc" );
+            dmiProduct
+                = cleanupForHostname( QString::fromLocal8Bit( modelFile.readAll().chopped( 1 ).simplified().data() ) );
+            if ( !dmiProduct.isEmpty() )
+            {
+                tried = true;
+                return dmiProduct;
+            }
         }
+
+        dmiProduct = QStringLiteral( "pc" );
         tried = true;
     }
     return dmiProduct;
@@ -386,7 +412,7 @@ makeLoginNameSuggestion( const QStringList& parts )
         }
     }
 
-    return USERNAME_RX.indexIn( usernameSuggestion ) != -1 ? usernameSuggestion : QString();
+    return usernameSuggestion.indexOf( USERNAME_RX ) != -1 ? usernameSuggestion : QString();
 }
 
 /** @brief Return an invalid string for use in a hostname, if @p s is empty
@@ -403,24 +429,24 @@ invalidEmpty( const QString& s )
 STATICTEST QString
 makeHostnameSuggestion( const QString& templateString, const QStringList& fullNameParts, const QString& loginName )
 {
-    QHash< QString, QString > replace;
+    Calamares::String::DictionaryExpander d;
     // User data
-    replace.insert( QStringLiteral( "first" ),
-                    invalidEmpty( fullNameParts.isEmpty() ? QString() : cleanupForHostname( fullNameParts.first() ) ) );
-    replace.insert( QStringLiteral( "name" ), invalidEmpty( cleanupForHostname( fullNameParts.join( QString() ) ) ) );
-    replace.insert( QStringLiteral( "login" ), invalidEmpty( cleanupForHostname( loginName ) ) );
-    // Hardware data
-    replace.insert( QStringLiteral( "product" ), guessProductName() );
-    replace.insert( QStringLiteral( "product2" ), cleanupForHostname( QSysInfo::prettyProductName() ) );
-    replace.insert( QStringLiteral( "cpu" ), cleanupForHostname( QSysInfo::currentCpuArchitecture() ) );
-    // Hostname data
-    replace.insert( QStringLiteral( "host" ), invalidEmpty( cleanupForHostname( QSysInfo::machineHostName() ) ) );
+    d.add( QStringLiteral( "first" ),
+           invalidEmpty( fullNameParts.isEmpty() ? QString() : cleanupForHostname( fullNameParts.first() ) ) )
+        .add( QStringLiteral( "name" ), invalidEmpty( cleanupForHostname( fullNameParts.join( QString() ) ) ) )
+        .add( QStringLiteral( "login" ), invalidEmpty( cleanupForHostname( loginName ) ) )
+        // Hardware data
+        .add( QStringLiteral( "product" ), guessProductName() )
+        .add( QStringLiteral( "product2" ), cleanupForHostname( QSysInfo::prettyProductName() ) )
+        .add( QStringLiteral( "cpu" ), cleanupForHostname( QSysInfo::currentCpuArchitecture() ) )
+        // Hostname data
+        .add( QStringLiteral( "host" ), invalidEmpty( cleanupForHostname( QSysInfo::machineHostName() ) ) );
 
-    QString hostnameSuggestion = KMacroExpander::expandMacros( templateString, replace, '$' );
+    QString hostnameSuggestion = d.expand( templateString );
 
     // RegExp for valid hostnames; if the suggestion produces a valid name, return it
-    static const QRegExp HOSTNAME_RX( "^[a-zA-Z0-9][-a-zA-Z0-9_]*$" );
-    return HOSTNAME_RX.indexIn( hostnameSuggestion ) != -1 ? hostnameSuggestion : QString();
+    static const QRegularExpression HOSTNAME_RX( "^[a-zA-Z0-9][-a-zA-Z0-9_]*$" );
+    return hostnameSuggestion.indexOf( HOSTNAME_RX ) != -1 ? hostnameSuggestion : QString();
 }
 
 void
@@ -457,15 +483,14 @@ Config::setFullName( const QString& name )
         emit fullNameChanged( name );
 
         // Build login and hostname, if needed
-        static QRegExp rx( "[^a-zA-Z0-9 ]", Qt::CaseInsensitive );
+        static QRegularExpression rx( "[^a-zA-Z0-9 ]" );
 
-        const QString cleanName = CalamaresUtils::removeDiacritics( transliterate( name ) )
-                                      .replace( QRegExp( "[-']" ), "" )
+        const QString cleanName = Calamares::String::removeDiacritics( transliterate( name ) )
+                                      .replace( QRegularExpression( "[-']" ), "" )
                                       .replace( rx, " " )
                                       .toLower()
                                       .simplified();
 
-
         QStringList cleanParts = cleanName.split( ' ' );
 
         if ( !m_customLoginName )
@@ -587,7 +612,6 @@ Config::passwordStatus( const QString& pw1, const QString& pw2 ) const
     return qMakePair( PasswordValidity::Valid, tr( "OK!" ) );
 }
 
-
 Config::PasswordStatus
 Config::userPasswordStatus() const
 {
@@ -608,7 +632,6 @@ Config::userPasswordMessage() const
     return p.second;
 }
 
-
 void
 Config::setRootPassword( const QString& s )
 {
@@ -715,7 +738,6 @@ Config::checkReady()
     }
 }
 
-
 STATICTEST void
 setConfigurationDefaultGroups( const QVariantMap& map, QList< GroupDescription >& defaultGroups )
 {
@@ -725,7 +747,7 @@ setConfigurationDefaultGroups( const QVariantMap& map, QList< GroupDescription >
     auto groupsFromConfig = map.value( key ).toList();
     if ( groupsFromConfig.isEmpty() )
     {
-        if ( map.contains( key ) && map.value( key ).isValid() && map.value( key ).canConvert( QVariant::List ) )
+        if ( map.contains( key ) && map.value( key ).isValid() && map.value( key ).canConvert< QVariantList >() )
         {
             // Explicitly set, but empty: this is valid, but unusual.
             cDebug() << key << "has explicit empty value.";
@@ -746,19 +768,19 @@ setConfigurationDefaultGroups( const QVariantMap& map, QList< GroupDescription >
     {
         for ( const auto& v : groupsFromConfig )
         {
-            if ( v.type() == QVariant::String )
+            if ( Calamares::typeOf( v ) == Calamares::StringVariantType )
             {
                 defaultGroups.append( GroupDescription( v.toString() ) );
             }
-            else if ( v.type() == QVariant::Map )
+            else if ( Calamares::typeOf( v ) == Calamares::MapVariantType )
             {
                 const auto innermap = v.toMap();
-                QString name = CalamaresUtils::getString( innermap, "name" );
+                QString name = Calamares::getString( innermap, "name" );
                 if ( !name.isEmpty() )
                 {
                     defaultGroups.append( GroupDescription( name,
-                                                            CalamaresUtils::getBool( innermap, "must_exist", false ),
-                                                            CalamaresUtils::getBool( innermap, "system", false ) ) );
+                                                            Calamares::getBool( innermap, "must_exist", false ),
+                                                            Calamares::getBool( innermap, "system", false ) ) );
                 }
                 else
                 {
@@ -777,7 +799,7 @@ STATICTEST HostNameAction
 getHostNameAction( const QVariantMap& configurationMap )
 {
     HostNameAction setHostName = HostNameAction::EtcHostname;
-    QString hostnameActionString = CalamaresUtils::getString( configurationMap, "location" );
+    QString hostnameActionString = Calamares::getString( configurationMap, "location" );
     if ( !hostnameActionString.isEmpty() )
     {
         bool ok = false;
@@ -812,18 +834,8 @@ addPasswordCheck( const QString& key, const QVariant& value, PasswordCheckList&
     }
     else if ( key == "nonempty" )
     {
-        if ( value.toBool() )
-        {
-            passwordChecks.push_back(
-                PasswordCheck( []() { return QCoreApplication::translate( "PWQ", "Password is empty" ); },
-                               []( const QString& s ) { return !s.isEmpty(); },
-                               PasswordCheck::Weight( 1 ) ) );
-        }
-        else
-        {
-            cDebug() << "nonempty check is mentioned but set to false";
-            return false;
-        }
+        cWarning() << "nonempty check is ignored; use minLength: 1";
+        return false;
     }
 #ifdef CHECK_PWQUALITY
     else if ( key == "libpwquality" )
@@ -862,75 +874,75 @@ either( T ( *f )( const QVariantMap&, const QString&, U ),
     }
 }
 
-// TODO:3.3: Remove
+/** @brief Tidy up a list of names
+ *
+ * Remove duplicates, apply lowercase, sort.
+ */
 static void
-copyLegacy( const QVariantMap& source, const QString& sourceKey, QVariantMap& target, const QString& targetKey )
+tidy( QStringList& l )
 {
-    if ( source.contains( sourceKey ) )
-    {
-        if ( target.contains( targetKey ) )
-        {
-            cWarning() << "Legacy *users* key" << sourceKey << "ignored.";
-        }
-        else
-        {
-            const QVariant legacyValue = source.value( sourceKey );
-            cWarning() << "Legacy *users* key" << sourceKey << "overrides hostname-settings.";
-            target.insert( targetKey, legacyValue );
-        }
-    }
+    std::for_each( l.begin(), l.end(), []( QString& s ) { s = s.toLower(); } );
+    l.sort();
+    l.removeDuplicates();
 }
 
 void
 Config::setConfigurationMap( const QVariantMap& configurationMap )
 {
-    QString shell( QLatin1String( "/bin/bash" ) );  // as if it's not set at all
-    if ( configurationMap.contains( "userShell" ) )
+    // Handle *user* key and subkeys and legacy settings
     {
-        shell = CalamaresUtils::getString( configurationMap, "userShell" );
+        bool ok = false;  // Ignored
+        QVariantMap userSettings = Calamares::getSubMap( configurationMap, "user", ok );
+
+        QString shell( QLatin1String( "/bin/bash" ) );  // as if it's not set at all
+        if ( userSettings.contains( "shell" ) )
+        {
+            shell = Calamares::getString( userSettings, "shell" );
+        }
+        // Now it might be explicitly set to empty, which is ok
+        setUserShell( shell );
+
+        m_forbiddenLoginNames = Calamares::getStringList( userSettings, "forbidden_names" );
+        m_forbiddenLoginNames << alwaysForbiddenLoginNames();
+        tidy( m_forbiddenLoginNames );
     }
-    // Now it might be explicitly set to empty, which is ok
-    setUserShell( shell );
 
     setAutoLoginGroup( either< QString, const QString& >(
-        CalamaresUtils::getString, configurationMap, "autologinGroup", "autoLoginGroup", QString() ) );
-    setSudoersGroup( CalamaresUtils::getString( configurationMap, "sudoersGroup" ) );
-    m_sudoStyle = CalamaresUtils::getBool( configurationMap, "sudoersConfigureWithGroup", false )
-        ? SudoStyle::UserAndGroup
-        : SudoStyle::UserOnly;
+        Calamares::getString, configurationMap, "autologinGroup", "autoLoginGroup", QString() ) );
+    setSudoersGroup( Calamares::getString( configurationMap, "sudoersGroup" ) );
+    m_sudoStyle = Calamares::getBool( configurationMap, "sudoersConfigureWithGroup", false ) ? SudoStyle::UserAndGroup
+                                                                                             : SudoStyle::UserOnly;
 
     // Handle *hostname* key and subkeys and legacy settings
     {
         bool ok = false;  // Ignored
-        QVariantMap hostnameSettings = CalamaresUtils::getSubMap( configurationMap, "hostname", ok );
+        QVariantMap hostnameSettings = Calamares::getSubMap( configurationMap, "hostname", ok );
 
-        // TODO:3.3: Remove calls to copyLegacy
-        copyLegacy( configurationMap, "setHostname", hostnameSettings, "location" );
-        copyLegacy( configurationMap, "writeHostsFile", hostnameSettings, "writeHostsFile" );
         m_hostnameAction = getHostNameAction( hostnameSettings );
-        m_writeEtcHosts = CalamaresUtils::getBool( hostnameSettings, "writeHostsFile", true );
+        m_writeEtcHosts = Calamares::getBool( hostnameSettings, "writeHostsFile", true );
         m_hostnameTemplate
-            = CalamaresUtils::getString( hostnameSettings, "template", QStringLiteral( "${first}-${product}" ) );
+            = Calamares::getString( hostnameSettings, "template", QStringLiteral( "${first}-${product}" ) );
+
+        m_forbiddenHostNames = Calamares::getStringList( hostnameSettings, "forbidden_names" );
+        m_forbiddenHostNames << alwaysForbiddenHostNames();
+        tidy( m_forbiddenHostNames );
     }
 
     setConfigurationDefaultGroups( configurationMap, m_defaultGroups );
 
     // Renaming of Autologin -> AutoLogin in 4ffa79d4cf also affected
     // configuration keys, which was not intended. Accept both.
-    m_doAutoLogin = either( CalamaresUtils::getBool,
-                            configurationMap,
-                            QStringLiteral( "doAutologin" ),
-                            QStringLiteral( "doAutoLogin" ),
-                            false );
+    m_doAutoLogin = either(
+        Calamares::getBool, configurationMap, QStringLiteral( "doAutologin" ), QStringLiteral( "doAutoLogin" ), false );
 
-    m_writeRootPassword = CalamaresUtils::getBool( configurationMap, "setRootPassword", true );
+    m_writeRootPassword = Calamares::getBool( configurationMap, "setRootPassword", true );
     Calamares::JobQueue::instance()->globalStorage()->insert( "setRootPassword", m_writeRootPassword );
 
-    m_reuseUserPasswordForRoot = CalamaresUtils::getBool( configurationMap, "doReusePassword", false );
+    m_reuseUserPasswordForRoot = Calamares::getBool( configurationMap, "doReusePassword", false );
 
-    m_permitWeakPasswords = CalamaresUtils::getBool( configurationMap, "allowWeakPasswords", false );
+    m_permitWeakPasswords = Calamares::getBool( configurationMap, "allowWeakPasswords", false );
     m_requireStrongPasswords
-        = !m_permitWeakPasswords || !CalamaresUtils::getBool( configurationMap, "allowWeakPasswordsDefault", false );
+        = !m_permitWeakPasswords || !Calamares::getBool( configurationMap, "allowWeakPasswordsDefault", false );
 
     // If the value doesn't exist, or isn't a map, this gives an empty map -- no problem
     auto pr_checks( configurationMap.value( "passwordRequirements" ).toMap() );
@@ -957,7 +969,7 @@ Config::finalizeGlobalStorage() const
     {
         gs->insert( "reuseRootPassword", reuseUserPasswordForRoot() );
     }
-    gs->insert( "password", CalamaresUtils::obscure( userPassword() ) );
+    gs->insert( "password", Calamares::String::obscure( userPassword() ) );
 }
 
 Calamares::JobList
